/*
  Copyright 2021 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.partner.friend.adapter

import android.view.LayoutInflater
import android.view.ViewGroup
import androidx.recyclerview.widget.RecyclerView
import com.kakao.sdk.partner.friend.imageloader.ImageLoader
import com.kakao.sdk.partner.friend.model.Item
import com.kakao.sdk.partner.friend.util.getDefaultProfileImage
import com.kakao.sdk.v2.partner.friend.databinding.KakaoSdkItemPickedFriendBinding

/**
 * @suppress
 */
internal class PickedFriendsAdapter(
    private val pickedFriends: MutableList<Item.User>,
    private val removeFriendCallback: (Item.User) -> Unit
) : RecyclerView.Adapter<PickedFriendsAdapter.PickedFriendViewHolder>() {

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): PickedFriendViewHolder {
        val inflater = LayoutInflater.from(parent.context)
        return PickedFriendViewHolder(
            KakaoSdkItemPickedFriendBinding.inflate(inflater, parent, false)
        ) {
            removePickedFriend(it)
            removeFriendCallback(it)
        }
    }

    override fun onBindViewHolder(holder: PickedFriendViewHolder, position: Int) {
        holder.bind(pickedFriends[position])
    }

    override fun getItemCount(): Int = pickedFriends.size

    fun addPickedFriend(friend: Item.User) {
        val isFirst = pickedFriends.isEmpty()
        pickedFriends.add(0, friend)

        // 첫 번째 친구 추가할 때 깜빡거리는 현상 제거
        if (isFirst) {
            notifyDataSetChanged()
        } else {
            notifyItemInserted(0)
        }
    }

    fun removePickedFriend(Friend: Item.User) {
        val index = pickedFriends.indexOf(Friend)
        // index를 못찾은 경우에는 -1 이므로 crash 방지
        if (index > -1) {
            pickedFriends.removeAt(index)
            notifyItemRemoved(index)
        }
    }

    class PickedFriendViewHolder(
        private val binding: KakaoSdkItemPickedFriendBinding,
        private val removeFriendCallback: (Item.User) -> Unit
    ) :
        RecyclerView.ViewHolder(binding.root) {

        fun bind(user: Item.User) {
            binding.apply {
                val defaultProfileImage =
                    getDefaultProfileImage(user.uuid.hashCode(), root.context).resourceId
                ImageLoader.get(itemView.context).load(user.profileImageUrl)
                    .placeHolder(defaultProfileImage).into(profileIv)
                pickedNicknameTv.text = user.nickname
                root.setOnClickListener {
                    removeFriendCallback(user)
                }
            }
        }
    }
}