/*
  Copyright 2021 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.partner.friend.imageloader

import android.graphics.BitmapFactory
import android.os.Handler
import android.widget.ImageView
import com.kakao.sdk.partner.friend.imageloader.ImageLoader.Companion.diskCache
import com.kakao.sdk.partner.friend.imageloader.model.DownloadResult
import com.kakao.sdk.partner.friend.imageloader.network.ImageDownloadService
import okhttp3.Cache
import okhttp3.OkHttpClient
import retrofit2.Retrofit

/**
 * @suppress
 */
internal class ImageDownloader(private val mainThreadHandler: Handler) {
    private val client by lazy {
        OkHttpClient.Builder().apply {
            cache(Cache(diskCache, calculateDiskCacheSize(diskCache)))
        }.build()
    }
    private val api =
        Retrofit.Builder().client(client).baseUrl(Constants.BASE_URL).build()
            .create(ImageDownloadService::class.java)

    fun downloadImage(target: ImageView, imageUrl: String) {
        val apiResult = api.downloadImage(imageUrl).execute()
        if (apiResult.isSuccessful) {
            apiResult.body()?.let {
                val bitmap = BitmapFactory.decodeStream(it.byteStream())
                val result = DownloadResult(bitmap, null, target)

                if (ImageLoader.loadCache(imageUrl) != bitmap) {
                    ImageLoader.saveCache(imageUrl, bitmap)
                }

                mainThreadHandler.run {
                    sendMessage(obtainMessage(Constants.IMAGE_DOWNLOAD_COMPLETE, result))
                }
            }
            return
        }
        val result = DownloadResult(null, apiResult.message(), target)
        mainThreadHandler.run {
            sendMessage(obtainMessage(Constants.IMAGE_DOWNLOAD_FAIL, result))
        }
    }
}